# Copyright Red Hat, Inc. and others. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from odltools.mdsal.models.model import Model

MODULE = "neutron"


def neutron(store, args):
    return Neutron(MODULE, store, args)


class Neutron(Model):
    CONTAINER = "neutron"
    NETWORKS = "networks"
    NETWORK = "network"
    PORTS = "ports"
    PORT = "port"
    ROUTERS = "routers"
    ROUTER = "router"
    TRUNKS = "trunks"
    TRUNK = "trunk"
    MODULE = "name"
    UUID = "uuid"

    def get_clist(self):
        return self.data[self.CONTAINER]

    def get_ccl(self, parent, child, item):
        c = self.data and self.data.get(parent, {})
        lst = self.get_list(c, child, item)
        return lst

    def get_ccl_by_key(self, parent, child, item, key="uuid"):
        d = {}
        lst = self.get_ccl(parent, child, item)
        for l in lst:
            d[l[key]] = l
        return d

    def get_networks_by_key(self, key="uuid"):
        return self.get_ccl_by_key(self.CONTAINER, self.NETWORKS, self.NETWORK, key)

    def get_ports_by_key(self, key="uuid"):
        return self.get_ccl_by_key(self.CONTAINER, self.PORTS, self.PORT, key)

    def get_trunks_by_key(self, key="uuid"):
        return self.get_ccl_by_key(self.CONTAINER, self.TRUNKS, self.TRUNK, key)
